;		     Ŀ
;                      Real-mode screen output procs  
;                          using BIOS functions       
;                    

EmitChar_RM	proc	far

	pushad
		mov	[Emit_DB], dl
		mov	si, offset Emit_DB
		call	EmitString_RM
	popad
		ret

EmitChar_RM	endp

ProcEmit_EAX_RM label	far

	pushad
                mov     cx, 8
		jmp	Emit_A_RM

ProcEmit_AX_RM	label	far

	pushad
                mov     cx, 4
                shl     eax, 16
		jmp	Emit_A_RM

Emit_A_RM	proc	far

Emit_A_RM_Loop:
                sub     dx, dx
                shld    edx, eax, 4     ;2^4=16=1 hex digit
                shl     eax, 4          ;SHLD does not shift EAX!

		.IF	dl >= 0Ah
                sub     dl, 0Ah
                add     dl, 'A'
		.ELSE
                add     dl, '0'
		.ENDIF

		call	EmitChar_RM
		loop	Emit_A_RM_Loop
                mov     dl, ' '
		call	EmitChar_RM
	popad
		ret

Emit_A_RM	endp

EmitString_RM	proc	far

        pushad
	push	DS
	push	ES
        pushf
                cld

EmitStringLoop_RM:

                lodsb
                or      al, al
		jz	EmitTerm_RM

		call	[EmitCharVector]

		jmp	EmitStringLoop_RM
EmitTerm_RM:

        popf
	pop	ES
	pop	DS
        popad
                ret

EmitString_RM	endp

EmitChar_BIOS	proc

;Emit character in AL.

;The BIOS Write TTY does not set the attribute, so first write a blank with
;the attribute using BIOS Write Char/Attrib.

		push	ax
		mov	cx, 1
		mov	bx, 7		;attribute
		mov	ax, 920H	;space
		int	10h		;BIOS write char/attrib
		pop	ax
                mov     ah, 0Eh         ;BIOS teletype
		int	10h

		ret

EmitChar_BIOS	endp

EmitChar_DOS	proc

;Emit character in AL.

		push	DS
		mov	bx, [Emit_DOS_Handle]
		mov	cx, 1
		push	ax
		mov	dx, sp
		mov	ax, SS
		mov	DS, ax
		mov	ah, 40h
		int	21h
		pop	ax
		pop	DS

		ret

EmitChar_DOS	endp

CRLF_RM 	proc	far

	pusha
	pushf
		.IF	[EmitCharVector] == offset EmitChar_BIOS

		zero	bh
		mov	dx, [CursorLoc]
                mov     ah, 2
                int     10h             ;set cursor location

		mov	ax, 0E0Dh
                int     10h             ;BIOS write TTY char
                mov     ax, 0E0Ah
                int     10h             ;BIOS write TTY char

		mov	ah, 3
                int     10h             ;get cursor location
		mov	[CursorLocY], dh
		mov	[CursorLocX], dl

		.ENDIF

		.IF	[EmitCharVector] == offset EmitChar_DOS

		mov	ax, 0A0Dh
		push	DS
		mov	bx, [Emit_DOS_Handle]
		mov	cx, 2
		push	ax
		mov	dx, sp
		mov	ax, SS
		mov	DS, ax
		mov	ah, 40h
		int	21h
		pop	ax
		pop	DS

		.ENDIF

	popf
	popa
		ret

CRLF_RM 	endp

CLS_RM		proc	far
	pusha
	push ES
		zero	ax
		mov	ES, ax
		mov	[CursorLoc], ax
		mov	bx, 449h
		mov	al, ES:[bx]		;AH=0 from above
		int	10h
	pop ES
	popa
		ret

CLS_RM		endp

Emit_AL_RM	proc	far

        pushad
		call	Emit_AL_Prim_RM
		call	CRLF_RM
        popad
		ret

Emit_AL_RM	endp

Emit_AX_RM	proc	far

        pushad
	push ax
                mov     al, ah
		call	Emit_AL_Prim_RM
	pop  ax
		call	Emit_AL_Prim_RM
		call	CRLF_RM
        popad
		ret

Emit_AX_RM	endp

EmitCharPrim_RM proc	far

	push ax

		.IF	dl <= 9
                add     dl, '0'
		.ELSE
                sub     dl, 0Ah
                add     dl, 'A'
		.ENDIF

		call	EmitChar_RM
        pop ax
                ret

EmitCharPrim_RM endp

Emit_AL_Prim_RM proc

;Only emits a byte value in hex, intended for emitting DOS error codes, in RM.

                zero    ah
                shl     ax, 4
                mov     dl, ah
		call	EmitCharPrim_RM
                zero    ah
                shl     ax, 4

                mov     dl, ah
		call	EmitCharPrim_RM

                ret

Emit_AL_Prim_RM endp

EmitString_CRLF_RM proc far

		call	EmitString_RM
		jmp	CRLF_RM

EmitString_CRLF_RM endp

ProcEmit_AX_CRLF_RM proc far

		call	ProcEmit_AX_RM
		jmp	CRLF_RM

ProcEmit_AX_CRLF_RM endp

ProcEmit_EAX_CRLF_RM proc far

		call	ProcEmit_EAX_RM
		jmp	CRLF_RM

ProcEmit_EAX_CRLF_RM endp

SyncCursorWithBIOS proc far

;-------------------------------------------------------------------------------
;Set cursor location to current BIOS cursor location.
;Use only 8086 instructions, unknown if CPU is a 386 at this point!
;-------------------------------------------------------------------------------

		mov	bh, 0				;Get Cursor Info
		mov	ah, 3
		int	10h				;DH=row DL=col
		mov	[CursorLoc], dx

		call	CRLF_RM

		ret

SyncCursorWithBIOS endp

SyncBIOSWithCursor proc far

		mov	dx, [CursorLoc]
		mov	bh, 0
		mov	ah, 2
		int	10h

		ret

SyncBIOSWithCursor endp
