
;
;			  Ŀ
;			      Stack Segment	
;                         
;

V86MonStackSeg	segment para stack use32 'STACK'

;----------------------------------
		db 100H dup(?)
Stack_V86:
;----------------------------------
		db 100H dup(?)
Stack_Scheduler_TSS:
;----------------------------------
Stack_Real_Bot	db 100H dup(?)
Stack_Real:						;also used as the initial TSS for LTR
;----------------------------------			;=startup TOS defined in EXE

V86MonStackSegEnd:
V86MonStackLimit:

V86MonStackSeg	ends

;
;			  Ŀ
;			      Data Segment    
;			  
;

V86MonDataSeg segment para public use32 'DATA'

Task_Scheduler_TaskSwitch	label fword
				dd 0			;offset not used
				dw Scheduler_TSS_SEL
ALIGN 4

;-----
Prefixes			label dword
Prefix_OperandOverride		db 0			;\
Prefix_AddressOverride		db 0			; \ group of 4 bytes, reset with 1 32-bit MOV
Prefix_SegmentOverride		db 0			; /
Prefix_Repeat			db 0			;/
;-----

TDB				dd offset TD_V86	;task defininition base (V86Mon is V86 TSS)
ErrorCode			dd ?			;error code pushed by the CPU from an exception
Vector				dd ?			;identifies the number of an exception

;Display
Emit_DB 			db ?,0			;char to emit, null
CursorLoc			label word
CursorLocX			db ?
CursorLocY			db ?
EmitCharVector			dw offset EmitChar_BIOS
Emit_DOS_Handle 		dw 1			;STDOUT

;Misc
CPU				db ?
NPX_Flag			db ?
CPUID_Implemented		db 0

temp				dd 5 dup(?)
tempb				equ <@8  temp>
tempw				equ <@16 temp>
tempd				equ <@32 temp>

UFO				db 0
UFO_Opcode			dw 0
UFO_CS				dd 0
UFO_EIP 			dd 0
UFO_EFL 			dd 0

Exception			db 0
Exception_CS			dd 0
Exception_EIP			dd 0

;Demo

SavedIntFF			dd 0
TSC_Start			dd 0, 0
TSC_End 			dd 0, 0
UnexpectedError 		db 0

;
;                     Ŀ
;                       Global Descriptor Table  
;                                 GDT            
;                     

ALIGN 10H

GDT			dq 0				;null descriptor

Code_SEL0		= ($ - GDT)
			dw offset16 V86MonCodeSeg32End
Thunk_Code0             dw 0
                        db 0
			db 10011010b			;readable non-conf PL0 code segment
			db 01000000b			;D=1 32-bit
                        db 0

Data_SEL0               = ($ - GDT)
			dw offset16 V86MonDataSegEnd	;limit
Thunk_Data0             dw 0
                        db 0
			db 10010010b			;writeable data segment
			db 01000000b			;D=1 byte granular
                        db 0

Stack_SEL0		= ($ - GDT)
			dw offset16 V86MonStackSegEnd	;limit
Thunk_Stack0            dw 0
                        db 0
			db 10010010b			;writeable data segment, ok as stack
			db 01000000b			;D=1 32-bit
                        db 0

Init_TSS_SEL		= ($ - GDT)
			dw 68H-1			;limit
Thunk_InitTSS		dw offset16 Stack_Real-68H	;Stack_Real is available when LTR is executed
                        db 0
			db 10001001b			;present, available 32-bit TSS
                        db 0
                        db 0

Scheduler_TSS_SEL	= ($ - GDT)
Scheduler_TSS_Desc	dw Scheduler_TSS_End - Scheduler_TSS
Thunk_SchedTSS		dw offset16 Scheduler_TSS
                        db 0
			db 10001001b			;present, available 32-bit TSS
                        db 0
                        db 0

Real_SEL		= ($ - GDT)
			dw 0FFFFh			;real mode 64k limit
			dw 0				;base arbitrary
                        db 0
			db 10010010b			;present, DPL0, read/write
			db 0				;byte granular
                        db 0

Real_Code_SEL		= ($ - GDT)			;Same as Code_SEL but with 64k limit
                        dw 0FFFFh
Thunk_RealCS            dw 0
                        db 0
			db 10011010b			;readable code segment
			db 0				;byte granular
                        db 0

FLAT_SEL3		= ($ - GDT) $RPL3
FLAT_SEL		EQU FLAT_SEL3

FLAT_Desc3		dw 0FFFFh			;max limit 15..00
			dw 0				;linear start = 0
                        db 0
			db 11110010b			;present, DPL3, r/w
			db 10001111b			;page granular, limit set 23-16
                        db 0

GDT_Limit = ($-1)-GDT

.errnz (GDT_Limit) GE 10000H,<GDT exceeds 64K bytes>
.errnz (GDT_Limit+1) mod 8,<Global descriptor table length not a multiple of 8>

;
;                   Ŀ
;                     Interrupt Descriptor Table  
;                               IDT               
;                   

ALIGN 10H

IDT_V86:						;Vector Type	ErrCode Intel definiton 	PC/DOS definition
	DPL3IntGateDesc Handler_V86_Interrupt_00	;00	Fault	No	divide error		-
	DPL0IntGateDesc Handler_V86_Debug		;01	F/T	No	debug			-
	DPL3IntGateDesc Handler_V86_Interrupt_02	;02	Fault	No	NMI			-
	DPL3IntGateDesc Handler_V86_Interrupt_03	;03	Trap	No	breakpoint		-
	DPL3IntGateDesc Handler_V86_Interrupt_04	;04	Trap	No	INTO			-
	DPL3IntGateDesc Handler_V86_Interrupt_05	;05	Fault	No	BOUNDS			INT 5|Shift+PrtScr print screen
	DPL0IntGateDesc Handler_V86_InvalidOpcode	;06	Fault	No	invalid opcode		-
	DPL0IntGateDesc Handler_V86_DeviceNotAvailable	;07	Fault	No	device not available	-
	DPL0IntGateDesc Handler_V86_DoubleFault_IRQ0	;08	Fault	Yes	double fault		IRQ0 system timer
	DPL3IntGateDesc Handler_V86_Interrupt_09	;09	Abort	No	NPX segment overrun	IRQ1 keyboard
	DPL3IntGateDesc Handler_V86_Interrupt_0A	;0A	Fault	Yes	invalid TSS		IRQ2 cascade
	DPL3IntGateDesc Handler_V86_Interrupt_0B	;0B	Fault	Yes	segment not present	IRQ3 serial port 2,4
	DPL3IntGateDesc Handler_V86_Interrupt_0C	;0C	Fault	Yes	stack fault		IRQ4 serial port 1,3
	DPL0IntGateDesc Handler_V86_GPF 		;0D	Fault	Yes	general protection	IRQ5 parallel port 2/network
	DPL0IntGateDesc Handler_V86_PageFault		;0E	Fault	Yes	page fault		IRQ6 diskette
	DPL3IntGateDesc Handler_V86_Interrupt		;0F	-	-	reserved		IRQ7 printer 1
	DPL3IntGateDesc Handler_V86_Interrupt_10	;10	Fault	No	floating-point error	BIOS video
	DPL3IntGateDesc Handler_V86_Interrupt_11	;11	Fault	Yes	alignment		BIOS equipment configuration

	include idtv86.inc

IDT_V86_End:

IDT_V86_Limit = ($-1)-IDT_V86

.errnz (IDT_V86_End-IDT_V86)-(256*8),<V86 interrupt descriptor table length not 256*8>

;Include table of GPF routes taken on faulting opcode.

include op1route.inc

.errnz ($-V86_GPF_OneByteOpcode_Routes)-(256*2),<There are not 256 GPF opcode routes.>

;

GDT_fword	label fword
		dw GDT_Limit
Thunk_GDT	dd offset GDT

IDT_V86_fword	label fword
		dw IDT_V86_Limit			;IDT limit
Thunk_IDT_V86	dd offset IDT_V86			;IDT linear address

IDT_Real_fword	label fword
		dw 256*4-1				;IDT limit (for 256 possible interrupts)
		dd 0					;IDT linear address (mapped to physical address 0)

;

;
comment %
                       Ŀ
		          Task State Segments	
		       
%

$IOBMBA textequ <offset16 IOBitmap - Scheduler_TSS>

ALIGN 4

;===============

Scheduler_TSS label dword
	dd 0					;back link (selector,0)
	dd 0					;ESP0  CPL0
	dd 0					;SS0   CPL0
	dd 0					;ESP   CPL1
	dd 0					;SS    CPL1
	dd 0					;ESP   CPL2
	dd 0					;SS    CPL2
	dd 0					;CR3 \
	dd offset Entry_Scheduler		;EIP  \
	dd 0					;EFL   \
	dd 0					;EAX	\
	dd 0					;ECX	 \
	dd 0					;EDX	  \
	dd 0					;EBX	   \
	dd offset Stack_Scheduler_TSS		;ESP	    \
	dd 0					;EBP	     > Saved to by outgoing task
	dd 0					;ESI	    /  Loaded from by incoming task
	dd 0					;EDI	   /
	dd Data_SEL0				;ES	  /
	dd Code_SEL0				;CS	 /
	dd Stack_SEL0				;SS	/
	dd Data_SEL0				;DS    /
	dd FLAT_SEL3				;FS   /
	dd FLAT_SEL3				;GS  /
	dd 0					;Task's LDT selector
	dw 0					;debug trap bit
	dw $IOBMBA				;I/O map base

IF _VME

;-------------------------------------------------------------------------------
;This is the interrupt bitmap. The interrupt bitmap is used to select
;8086 or protect-mode interrupt semantics for the INT imm8 instruction.
;-------------------------------------------------------------------------------

include intmap0.inc

ENDIF

IOBitmap db 8192 dup(0) 			;0=permit 1=forbid
IOBitmapEnd = $

Scheduler_TSS_End = $

;===============

Struct_TD_V86 STRUCT
			;---------------------------------------
 _BackLink		dd 0					;back link (selector,0)
 _ESP0			dd 0					;ESP0  CPL0
 _SS0			dd 0					;SS0   CPL0
 _ESP1			dd 0					;ESP   CPL1
 _SS1			dd 0					;SS    CPL1
 _ESP2			dd 0					;ESP   CPL2
 _SS2			dd 0					;SS    CPL2
 _CR3			dd 0					;CR3 \
 _EIP			dd 0					;EIP  \
 _EFL			dd 0					;EFL   \
 _EAX			dd 0					;EAX	\
 _ECX			dd 0					;ECX	 \
 _EDX			dd 0					;EDX	  \
 _EBX			dd 0					;EBX	   \
 _ESP			dd 0					;ESP	    \
 _EBP			dd 0					;EBP	     > Saved to by outgoing task
 _ESI			dd 0					;ESI	    /  Loaded from by incoming task
 _EDI			dd 0					;EDI	   /
 _ES			dd 0					;ES	  /
 _CS			dd 0					;CS	 /
 _SS			dd 0					;SS	/
 _DS			dd 0					;DS    /
 _FS			dd 0					;FS   /
 _GS			dd 0					;GS  /
 _LDT			dd 0					;Task's LDT selector
 _DebugT		dw 0					;debug trap bit
 _IOBM			dw 0					;I/O map base
			;------------- TSS extensions ----------
 LinearBase_SEL 	dd 0
 SetTrap_STI		db 0
 SetTrap_INS_OUTS	db 0
 Unadjusted_EIP 	dd 0
 Permitted_Port 	dw 0
 PIC1_PIC2_Command	db 0, 0
 _IF			dd 0
 Virtual_IF		dd 0
 STRUCT VPIC
   IRR			dw 0
   ISR			dw 0
   Vector               db 10H dup(0)
 ENDS
 HLT_Flag		db 0
 HLT_EIP		dd 0					;points past HLT
 HLT_CS 		dd 0
 STI_EFLAGS_TF		dd 0
 Recorded_EFLAGS	dd 0
			;---------------------------------------
Struct_TD_V86 ENDS

;---------------

			;---------------------------------------
TD_V86			dd 0					;back link (selector,0)
			dd offset Stack_V86_CPL0		;ESP0  CPL0
			dd Data_SEL0				;SS0   CPL0
			dd 0					;ESP   CPL1
			dd 0					;SS    CPL1
			dd 0					;ESP   CPL2
			dd 0					;SS    CPL2
			dd 0					;CR3 \
			dd offset BeginV86Mode			;EIP  \
			dd _Init_V86_EFLAGS			;EFL   \
			dd 0					;EAX	\
			dd 0					;ECX	 \
			dd 0					;EDX	  \
			dd 0					;EBX	   \
			dd offset Stack_V86			;ESP	    \
			dd 0					;EBP	     > Saved to by outgoing task
			dd 0					;ESI	    /  Loaded from by incoming task
			dd 0					;EDI	   /
			dd V86MonDataSeg			;ES	  /
			dd V86MonCodeSeg16			;CS	 /
			dd V86MonStackSeg			;SS	/
			dd V86MonDataSeg			;DS    /
			dd 0					;FS   /
			dd 0					;GS  /
			dd 0					;Task's LDT selector
			dw 0					;debug trap bit
			dw $IOBMBA				;I/O map base
			;------------- TSS extensions ----------
			dd FLAT_SEL3				;LinearBase_SEL
			db 0					;SetTrap_STI
			db 0					;SetTrap_INS_OUTS
			dd 0					;Unadjusted_EIP
			dw 0					;Permitted_Port
			db 0, 0 				;PIC1_PIC2_Command
			dd 0					;_IF
			dd _Virtual_IF				;Virtual_IF
								;STRUCT VPIC
			dw 0					;  IRR
			dw 0					;  ISR
                        db 10H dup(0)                           ;  Vector
			db 0					;HLT_Flag
			dd 0					;HLT_EIP  (points past HLT)
			dd 0					;HLT_CS
			dd 0					;STI_EFLAGS_TF
			dd 0					;Recorded_EFLAGS
			;---------------------------------------
			ALIGN 10H
			db 100H dup(?)
			Stack_V86_CPL0 = $
			;---------------------------------------
TD_V86_End     = $
;---------------

V86MonDataSeg	ends
